@@ -1,20 +1,20 @@
 /*************************************************************************************
  *  Copyright (C) 2012 by Alejandro Fiestas Olivares <afiestas@kde.org>              *
- *  Copyright (C) 2012 by Dan Vrátil <dvratil@redhat.com>                            *
+ *  Copyright (C) 2012, 2013 by Daniel Vrátil <dvratil@redhat.com>                   *
  *                                                                                   *
- *  This program is free software; you can redistribute it and/or                    *
- *  modify it under the terms of the GNU General Public License                      *
- *  as published by the Free Software Foundation; either version 2                   *
- *  of the License, or (at your option) any later version.                           *
+ *  This library is free software; you can redistribute it and/or                    *
+ *  modify it under the terms of the GNU Lesser General Public                       *
+ *  License as published by the Free Software Foundation; either                     *
+ *  version 2.1 of the License, or (at your option) any later version.               *
  *                                                                                   *
- *  This program is distributed in the hope that it will be useful,                  *
+ *  This library is distributed in the hope that it will be useful,                  *
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of                   *
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                    *
- *  GNU General Public License for more details.                                     *
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU                *
+ *  Lesser General Public License for more details.                                  *
  *                                                                                   *
- *  You should have received a copy of the GNU General Public License                *
- *  along with this program; if not, write to the Free Software                      *
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA   *
+ *  You should have received a copy of the GNU Lesser General Public                 *
+ *  License along with this library; if not, write to the Free Software              *
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA       *
  *************************************************************************************/
 
 #include "xrandroutput.h"
@@ -47,6 +47,8 @@ XRandROutput::XRandROutput(int id, bool primary, XRandRConfig *config)
     updateModes(outputInfo);
     fetchType();
     m_primary = primary;
+    m_widthMm = outputInfo->mm_width;
+    m_heightMm = outputInfo->mm_height;
 
     XRRFreeOutputInfo(outputInfo);
 }
@@ -82,11 +84,26 @@ QPoint XRandROutput::position() const
     return m_position;
 }
 
+XRandRMode::Map XRandROutput::modes() const
+{
+    return m_modes;
+}
+
 QString XRandROutput::currentModeId() const
 {
     return m_currentMode;
 }
 
+XRandRMode* XRandROutput::currentMode() const
+{
+    int modeId = m_currentMode.toInt();
+    if (!m_modes.contains(modeId)) {
+        return 0;
+    }
+
+    return m_modes[modeId];
+}
+
 KScreen::Output::Rotation XRandROutput::rotation() const
 {
     return m_rotation;
@@ -111,6 +128,11 @@ KScreen::Edid *XRandROutput::edid() const
 void XRandROutput::update(PrimaryChange primary)
 {
     XRROutputInfo *outputInfo = XRandR::XRROutput(m_id);
+    if (!outputInfo) {
+        Q_EMIT outputRemoved(m_id);
+        deleteLater();
+        return;
+    }
 
     m_changedProperties = 0;
     updateOutput(outputInfo);
@@ -164,7 +186,7 @@ void XRandROutput::updateOutput(const XRROutputInfo *outputInfo)
         }
 
         if (crtcInfo->mode) {
-            if (m_currentMode != QString::number((int) crtcInfo->mode)) {
+            if (m_currentMode != QString::number(crtcInfo->mode)) {
                 m_currentMode = QString::number(crtcInfo->mode);
                 m_changedProperties |= PropertyCurrentMode;
             }
@@ -200,6 +222,8 @@ void XRandROutput::updateModes(const XRROutputInfo *outputInfo)
     XRRScreenResources *resources = XRandR::screenResources();
 
     m_preferredModes.clear();
+    qDeleteAll(m_modes);
+    m_modes.clear();
     for (int i = 0; i < outputInfo->nmode; ++i)
     {
         /* Resources->modes contains all possible modes, we are only interested
@@ -327,6 +351,7 @@ KScreen::Output *XRandROutput::toKScreenOutput(KScreen::Config *parent) const
     m_changedProperties = 0;
     kscreenOutput->setId(m_id);
     kscreenOutput->setType(m_type);
+    kscreenOutput->setSizeMm(QSize(m_widthMm, m_heightMm));
     updateKScreenOutput(kscreenOutput);
 
     return kscreenOutput;
